<?php
/**
 * EPL Functions: Spam Titan API
 *
 * @link http://www.egroupware.org
 * @package epl
 * @subpackage api
 * @author Hadi Nategh <hn@egroupware.org>
 * @copyright (c) 2016 by EGroupware GmbH
 * @version $Id$
 */

class stylite_spamtitan
{
	/**
	 * default egroupware's SpamTitan API Url
	 */
	const DEFAULT_API_URL = 'https://mailgateway.egroupware.org';

	/**
	 * Default
	 * @var type
	 */
	private $data = array();

	function __construct($data) {
		$this->data = array_merge(array(
			'api_url'	=> self::DEFAULT_API_URL,
			'server'	=> '10.40.8.129',
			'imap'		=> 'imap.egroupware.org',
			'imap_port'	=> 993
		), $data);
	}
	 /**
	 * Make a call to SpamTitan API
	 *
	 * @param string $path path of api-url eg. '/domaingroup/add'
	 * @param array $params get parameters for api-call
	 * @param string $url protocol and domain for API eg. 'https://mailgateway.egroupware.org'
	 * @return SimpleXMLElement unserialized XML response
	 * @throws Exception if no valid response is received from SpamTitan
	 *
	 * @link http://download.spamtitan.com/manuals/SpamTitanAPIGuideV2.pdf
	 */
	public function api_call($path, array $params, $url=self::DEFAULT_API_URL)
	{
		$url = !empty($this->data['api_url']) ? $this->data['api_url'] : self::DEFAULT_API_URL;
		$url .= $path;

		foreach($params as $name => &$value)
		{
			$value = urlencode($name).'='.strtr(urlencode($value), array(
				// Spamtitian does not decode these :(
				'%7B' => '{',
				'%3A' => ':',
				'%7D' => '}',
			));
		}
		$params = array_merge ($params, array(
			'userpwd' => 'userpwd='.$this->data['userpwd'],
			'user' => 'user='.$this->data['user'],
			'stoa' => 'stoa='.$this->data['stoa']
		));
		$url .= '?'.implode('&', $params);

		if (!($xml = file_get_contents($url)))
		{
			throw new Exception("Error contacting SpamTitan: $url");
		}
		return new SimpleXMLElement($xml);
	}

	/**
	 * Learn email as Spam or Ham
	 *
	 * @param string $_mailbody Required. Email content
	 * @param string $_type = Spam, type of message
	 *		- Types: Spam | Ham
	 * @return string|boolean returns true if learning has been successful otherwise false
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function learnmail($_mailbody, $_type = 'Spam')
	{
		$url = (!empty($this->data['api_url'])? $this->data['api_url']: self::DEFAULT_API_URL).'/bayes/learnmail/'.
				'?user='.$this->data['user'].'&userpwd='.$this->data['userpwd'];
		$postdata = http_build_query(
			array(
				'mail_body' => urlencode($_mailbody),
				'type' => $_type
			)
		);
		$opts = array('http' =>
			array(
				'method'  => 'POST',
				'header'  => 'Content-type: application/x-www-form-urlencoded',
				'content' => $postdata
			)
		);
		$context  = stream_context_create($opts);

		if (!($xml = file_get_contents($url, false, $context))) throw new Exception("Error contacting SpamTitan");
		$resp =  new SimpleXMLElement($xml);
		return $resp['stat'] == 'ok' ? true: $resp->error[0];
	}

	/**
	 * List all the domains relayed by SpamTitan
	 *
	 * @param string $_server Optional. List Domain’s based on the IP or FQDN.
	 * @return array of domain-names or empty array if server is not found
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function list_domains($_server)
	{
		$resp = $this->api_call('/domain/list', array('server' => $_server));

		return $resp['stat'] == 'ok' ? (array)$resp->Domains->Domain : array();
	}

	/**
	 * Adds the specified domain to the current list of domains relayed
	 * by SpamTitan to the specified destination server.
	 *
	 * @param string $_name Required. The name of the Domain to
	 *  add. MX Lookup is automatically enabled for new domains.
	 *
	 * @param type $_server Required. The destination mail server
	 *  for this domain. To specify a non-default port
	 *  (25) append ":8025".
	 *	- To specify a fallback server enter a list of IP addresses/FQDN’s separated by spaces.
	 *		SpamTitan will attempt delivery in the order listed. E.g.: “server=1.2.3.4 5.6.2.3”
	 *	- Specify a comma (,) separated list of IP addresses and/or FQDN’s. In this
	 *		case SpamTitan will attempt delivery in a round-robin fashion between the
	 *		listed servers. E.g.:”server=1.2.3.4,1.2.3.5”
	 * @param string $_domaingroup Optional. The Domain Group to add this domain to
	 *
	 * @return boolean|string return true if successful otherwise error message
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function add_domain ($_name, $_server, $_domaingroup)
	{
		$resp = $this->api_call('/domain/add', array('name' => $_name,'server' => $_server, 'domaingroup' => $_domaingroup));

		return $resp['stat'] == 'ok' ? true : $resp->error[0];
	}

	/**
	 * Tests if the specified domain is been relayed by SpamTitan
	 *
	 * @param string $_name Required. The name of the Domain to add
	 * @return boolean|string return true if successful otherwise error message
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function domain_exists ($_name)
	{
		$resp = $this->api_call('/domain/exists', array('name' => $_name));

		return $resp['stat'] == 'ok' ? true : $resp->error[0];
	}

	/**
	 * Returns extended information of a given domain, specified by
	 * domain name as per the required name parameter below.
	 *
	 * @param string $_name Required. The name of the Domain to add
	 * @return array return information regarding the requested domain
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function show_domain ($_name)
	{
		$resp = $this->api_call('/domain/show', array('name' => $_name));

		return $resp['stat'] == 'ok' ? $resp->Data : array();
	}

	/**
	 * Delete a domain. Note that this will delete all user policies
	 * associated with this domain.
	 *
	 * @param string $_name Required. The name of the domain to delete.
	 * @return boolean|string return true if successful otherwise error message
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function delete_domain ($_name)
	{
		$resp = $this->api_call('/domain/delete', array('name' => $_name));

		return $resp['stat'] == 'ok' ? true : $resp->error[0];
	}

	/**
	 * Tests if the specified email address/domain is blacklisted
	 * by the specified user/domain, or globally if no user/domain is specified.
	 *
	 * @param string $_sender Required. The email address or domain to
	 *  check for the existence of in a particular blacklist
	 *
	 * @param string $_user Optional. The email address or domain name
	 *  of the blacklist to check. If no user is specified, then the
	 *  global blacklist will be checked
	 *
	 * @return boolean|string return true if successful otherwise error message
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function check_blacklist ($_sender, $_user)
	{
		$resp = $this->api_call('/blacklist/exists', array('sender' => $_sender, 'user' => $_user));

		return $resp['stat'] == 'ok' ? true : $resp->error[0];
	}

	/**
	 * Add an email address or domain to a specified users or domains
	 * blacklist. If no user/domain is specified then the entry is added
	 * to the global blacklist.
	 *
	 * @param string $_sender Required. The email address or domain to
	 *  add to the blacklist
	 *
	 * @param string $_user Optional. The email address or domain name
	 *  of the blacklist entry to add. If no user is specified, then the
	 *  entry will be added to the global blacklist.
	 *
	 * @return boolean|string return true if successful otherwise error message
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function add_blacklist_entry ($_sender, $_user)
	{
		$resp = $this->api_call('/blacklist/add', array('sender' => $_sender, 'user' => $_user));

		return $resp['stat'] == 'ok' ? true : $resp->error[0];
	}

	/**
	 * Delete an entry from the specified users/domains blacklist.
	 * If no user is specified, then delete the entry from the global blacklist.
	 *
	 * @param string $_sender Required. The email address or domain
	 * to delete from the blacklist
	 *
	 * @param string $_user Optional. The email address or domain name
	 * of the owner of the blacklist entry to delete. If no user is specified,
	 * then the entry will be deleted from the global blacklist.
	 *
	 * @return boolean|string return true if successful otherwise error message
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function delete_blacklist_entry ($_sender, $_user)
	{
		$resp = $this->api_call('/blacklist/delete', array('sender' => $_sender, 'user' => $_user));

		return $resp['stat'] == 'ok' ? true : $resp->error[0];
	}

	/**
	 * List all Whitelisted email addresses and/or domains for the
	 * specified user/domain. If no user or domain is specified then
	 * the global Whitelist will be returned.
	 *
	 * @param string $_user Optional. The email address or domain name
	 * of the user/domain for which to view the whitelist
	 *
	 * @return array return success response or false
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function show_whitelist ($_user)
	{
		$resp = $this->api_call('/whitelist/exists', array('sender' => $_sender, 'user' => $_user));

		return $resp['stat'] == 'ok' ? $resp->Whitelist : array();
	}

	/**
	 * Tests if the specified email address/domain is whitelisted by
	 * the specified user/domain, or globally if no user/domain is specified.
	 *
	 * @param string $_sender Required. The email address or domain
	 * to check for the existence of in a particular whitelist
	 *
	 * @param string $_user Optional.  The email address or domain name
	 * of the whitelist to check. If no user is specified, then the global
	 * whitelist will be checked
	 *
	 * @return boolean|string return true if successful otherwise error message
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function check_whitelist ($_sender, $_user)
	{
		$resp = $this->api_call('/whitelist/exists', array('sender' => $_sender, 'user' => $_user));

		return $resp['stat'] == 'ok' ? true : $resp->error[0];
	}

	/**
	 * Add an email address or domain to a specified users or domains
	 * whitelist. If no user/domain is specified then the entry is
	 * added to the global whitelist.
	 *
	 * @param string $_sender Required. The email address or domain to
	 * add to the whitelist
	 *
	 * @param string $_user Optional. The email address or domain name
	 *  of the whitelist entry to add. If no user is specified, then the
	 *  entry will be added to the global whitelist.
	 *
	 * @return boolean|string return true if successful otherwise error message
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function add_whitelist_entry ($_sender, $_user)
	{
		if (!empty($_user)) $this->data['user'] = $_user;
		$resp = $this->api_call('/whitelist/add', array('sender' => $_sender, 'user' => $_user));

		return $resp['stat'] == 'ok' ? true : $resp->error[0];
	}

	/**
	 * Delete an entry from the specified users/domains whitelist.
	 * If no user is specified, then delete the entry from the global whitelist.
	 *
	 * @param string $_sender Required. The email address or domain
	 * to delete from the whitelist
	 *
	 * @param string $_user Optional. The email address or domain name
	 * of the owner of the whitelist entry to delete. If no user is specified,
	 * then the entry will be deleted from the global whitelist.
	 *
	 * @return boolean|string return true if successful otherwise error message
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function delete_whitelist_entry ($_sender, $_user)
	{
		$resp = $this->api_call('/whitelist/delete', array('sender' => $_sender, 'user' => $_user));

		return $resp['stat'] == 'ok' ? true : $resp->error[0];
	}

	/**
	 * Add user policy. The policy which is added will be the same policy as the
	 * domain policy. It can subsequently be modified using the edit API.
	 *
	 * @param string $_name Required. The email address of the policy to add
	 *
	 * @return boolean|string return true if successful otherwise error message
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function add_policy ($_name)
	{
		$resp = $this->api_call('/policy/add', array('name' => $_name));

		return $resp['stat'] == 'ok' ? true : $resp->error[0];
	}

	/**
	 * Returns if the specified email address or domain policy exists.
	 *
	 * @param string $_name Required. The email address or domain of the policy
	 * to check for.
	 *
	 * @return boolean|string return true if successful otherwise error message
	 * @throws Exception if no valid response is received from SpamTitan
	 */
	public function policy_exists ($_name)
	{
		$resp = $this->api_call('/policy/exists', array('name' => $_name));

		return $resp['stat'] == 'ok' ? true : $resp->error[0];
	}
}